#include "std_precomp.h"

//ideas for new templates:
//something that uses different particles at once
//a simple 3d template
//something that provides for particle/particle interactions?
//maybe a lightning and water thing?

#include <wx/choice.h>
#include <wx/spinctrl.h>
#include <wx/textctrl.h>
#include <wx/dir.h>
#include "ifl_wx.h"
#include <particle.h>
#include <simple_regexp.h>




//I continue to debate whether or not these really ought to be defines...
const int ID_TWEAK_AGE = 500;  
const int ID_TWEAK_EMMIT_RATE = 501; 
const int ID_TWEAK_FORCE =  502;
const int ID_TWEAK_CENTER_BEHAVIOR = 503;

const int ID_CHOOSE_PERSP = 504;

const int PARAM_LIFESPAN = 505;
const int PARAM_EMIT_RATE = 506;
const int PARAM_SPEED = 507;
const int PARAM_FORCE = 508;
const int PARAM_DIR = 509;


const int CM_EDIT_T = 513;
const int CM_EDIT_R = 514;
const int CM_EDIT_G = 515;
const int CM_EDIT_B = 516;
const int CM_EDIT_A = 517;
const int CM_NEW_T = 518;
const int CM_NEW_R = 519;
const int CM_NEW_G = 520;
const int CM_NEW_B = 521;
const int CM_NEW_A = 522;
const int CM_COLOR_SEL = 510;
const int CM_COLOR_DELETE = 511;
const int CM_COLOR_ADD = 512;
const int CM_COLOR_EDIT = 523;

wxMenu * StandardEmitterMenu() {
  wxMenu *tweakMenu = new wxMenu;
  
  tweakMenu->Append(ID_EXPORT_SPE, "&Export Particle Info");
  tweakMenu->Append(ID_IMPORT_SPE, "&Import Particle Info");
  tweakMenu->Append(ID_CHANGE_PERSP, "&Change Perspective Matrix");
  tweakMenu->Append(ID_TWEAK_BITMAP, "&Change Bitmap");
  tweakMenu->Append(ID_TWEAK_PARAMS, "&Change Paramaters");
  tweakMenu->Append(ID_TWEAK_COLOR_MAP, "&Change Color Map");
  return tweakMenu;
}

///////////////////////////////////////////////////////
//Change Perspective Matrix stuff:
////////////////////////

class PerspMatrixDialog : public wxDialog {
	TestGLCanvas * glFrame;
public:
	PerspMatrixDialog(wxWindow * parent, TestGLCanvas * glFrame) :
	  glFrame(glFrame),
	  wxDialog(parent,-1,"Change Persp Matrix", wxDefaultPosition, wxSize(150,100)) {
	
		
		wxChoice * perspMatrix = new wxChoice(this, ID_CHOOSE_PERSP, wxDefaultPosition, 
			wxSize(100,20),2, PERSP_CHOICES);

		Show();	  
	  }

	  void OnChoose(wxCommandEvent &event) {
			glFrame->setPersp(event.GetSelection());
	  }

DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(PerspMatrixDialog, wxDialog)
	EVT_CHOICE(ID_CHOOSE_PERSP, PerspMatrixDialog::OnChoose)
END_EVENT_TABLE()

void MyFrame::ChangePersp(wxCommandEvent& event)
{
    (void) new PerspMatrixDialog(this,m_canvas);
}


///////////////////////////////////////////////////////
//ColorMap stuff:
////////////////////////

/*
//the float4's are in the range 0-255
struct cmap_point {
	float first;
	float4 second;
	bool operator<(const cmap_point &b) {
		return first<b.first;
	}
	cmap_point(float f, float4 color) : first(f), second(color) {}
};

struct Linear_Color_Map {
	vector<cmap_point> points;
*/


class ColorMapDialog : public wxDialog {
	effects::StandardEmitter<> * emitter;

	//so the general idea is that this thing maintains a list of color points.
	//you call loadColorMap, and that point list is brought into accord with
	//this color map's list.  call setcolormap, and we will try to force our
	//color point list onto the colormap, then reload.  Note that the color map
	//will enforce some normilization and sorting rules on the point list, so
	//calling setColorMap may change the values in our own point list.
	//otherwise it is just a normal list box filled with a point list representation.
	struct ColorMapBox : public wxListBox {
		effects::Linear_Color_Map * map;
		vector<effects::cmap_point> points;

		void loadColorMap() {
			//points.clear();
			points=map->get_points();
			Clear();
			for(vector<effects::cmap_point>::iterator i=points.begin();i!=points.end();i++) {
				char buffer[200];
				::sprintf(buffer,"%#03.2f :: (%03d , %03d , %03d, %03d)", i->first, (int)i->second[0], (int)i->second[1], (int)i->second[2], (int)i->second[3]);
				Append(buffer);
			}
		}

		void setColorMap() {
			*map=effects::Linear_Color_Map(points);
			loadColorMap();
		}

		ColorMapBox(effects::Linear_Color_Map * map, wxWindow * parent, int id) : map(map), 
			wxListBox(parent, id, wxPoint(0,10), wxSize(200,100)) {
			loadColorMap();
		}

	};

	struct FloatCtrl : public wxTextCtrl {
		float * fp;
		FloatCtrl(float * fp, wxWindow* parent, wxWindowID id, const wxPoint& pos, 
			const wxSize& size) : fp(fp), wxTextCtrl(parent,id, "", pos, size, wxTE_PROCESS_ENTER ) {
			(*this) << *fp;
		}

		void OnEnter() {
			using namespace boost_ex;
			string v(GetValue().c_str());
			if( is_float(v) ) {
				*fp=make_float(v);
			}
			else {
				SetValue("");
				(*this) << *fp;
			}
		}
	};

	struct Int_to_FloatCtrl : public wxTextCtrl {
		float * fp;
		Int_to_FloatCtrl(float * fp, wxWindow* parent, wxWindowID id, const wxPoint& pos, 
			const wxSize& size) : fp(fp), wxTextCtrl(parent,id, "", pos, size, wxTE_PROCESS_ENTER ) {
			(*this) << (int)*fp;
		}

		void OnEnter() {
			using namespace boost_ex;
			string v(GetValue().c_str());
			if( is_int(v) ) {
				*fp=make_int(v);
			}
			else {
				SetValue("");
				(*this) << (int)*fp;
			}
		}
	};

	effects::cmap_point edit_point;
	FloatCtrl * edit_t;
	Int_to_FloatCtrl * edit_r;
	Int_to_FloatCtrl * edit_g;
	Int_to_FloatCtrl * edit_b;
	Int_to_FloatCtrl * edit_a;

	effects::cmap_point new_point;
	FloatCtrl * new_t;
	Int_to_FloatCtrl * new_r;
	Int_to_FloatCtrl * new_g;
	Int_to_FloatCtrl * new_b;
	Int_to_FloatCtrl * new_a;

	int current_sel;
	ColorMapBox * colorbox;
public:
	ColorMapDialog(wxWindow * parent, effects::StandardEmitter<> * emitter) :
	  emitter(emitter), wxDialog(parent,-1,"Edit Color Map", wxPoint(0,10), wxSize(400,300)) {
		colorbox= new ColorMapBox(&emitter->color_fun, this, CM_COLOR_SEL);

		new wxButton(this, CM_COLOR_ADD, "Add Color Point", wxPoint(210,10), wxSize(100,-1) );
		new wxButton(this, CM_COLOR_EDIT, "Change Color Point", wxPoint(210,40), wxSize(100,-1) );
		new wxButton(this, CM_COLOR_DELETE, "Delete Color Point", wxPoint(210,70), wxSize(100,-1) );
		
		new wxStaticBox(this, -1, "New Color Point Values", wxPoint(0,120), wxSize(300,40));
		
		new wxStaticText(this, -1, "P:", wxPoint(3,139));
		new_t = new FloatCtrl(&new_point.first,this,CM_NEW_T,wxPoint(15,135), wxSize(40,-1));
		new wxStaticText(this, -1, "R:", wxPoint(63,139));
		new_r = new Int_to_FloatCtrl(&new_point.second[0],this,CM_NEW_R,wxPoint(75,135), wxSize(40,-1));
		new wxStaticText(this, -1, "G:", wxPoint(123,139));
		new_g = new Int_to_FloatCtrl(&new_point.second[1],this,CM_NEW_G,wxPoint(135,135), wxSize(40,-1));
		new wxStaticText(this, -1, "B:", wxPoint(183,139));
		new_b = new Int_to_FloatCtrl(&new_point.second[2],this,CM_NEW_B,wxPoint(195,135), wxSize(40,-1));
		new wxStaticText(this, -1, "A:", wxPoint(243,139));
		new_a = new Int_to_FloatCtrl(&new_point.second[3],this,CM_NEW_A,wxPoint(255,135), wxSize(40,-1));


		new wxStaticBox(this, -1, "Selected Color Point Values", wxPoint(0,170), wxSize(300,40));

		new wxStaticText(this, -1, "P:", wxPoint(3,189));
		edit_t = new FloatCtrl(&edit_point.first,this,CM_EDIT_T,wxPoint(15,185), wxSize(40,-1));
		new wxStaticText(this, -1, "R:", wxPoint(63,189));
		edit_r = new Int_to_FloatCtrl(&edit_point.second[0],this,CM_EDIT_R,wxPoint(75,185), wxSize(40,-1));
		new wxStaticText(this, -1, "G:", wxPoint(123,189));
		edit_g = new Int_to_FloatCtrl(&edit_point.second[1],this,CM_EDIT_G,wxPoint(135,185), wxSize(40,-1));
		new wxStaticText(this, -1, "B:", wxPoint(183,189));
		edit_b = new Int_to_FloatCtrl(&edit_point.second[2],this,CM_EDIT_B,wxPoint(195,185), wxSize(40,-1));
		new wxStaticText(this, -1, "A:", wxPoint(243,189));
		edit_a = new Int_to_FloatCtrl(&edit_point.second[3],this,CM_EDIT_A,wxPoint(255,185), wxSize(40,-1));


		Show();	  
	  }

	  void EditColor(wxCommandEvent &event) {
		int s=colorbox->GetSelection();
		if(s>=0) {
			edit_t->OnEnter();
			edit_r->OnEnter();
			edit_g->OnEnter();
			edit_b->OnEnter();
			edit_a->OnEnter();
			colorbox->points[s]=edit_point;
			colorbox->setColorMap();
		}
		colorbox->SetSelection(s);
	  }

	  void New_T(wxCommandEvent &event) { new_t->OnEnter(); }
	  void New_R(wxCommandEvent &event) { new_r->OnEnter(); }
	  void New_G(wxCommandEvent &event) { new_g->OnEnter(); }
	  void New_B(wxCommandEvent &event) { new_b->OnEnter(); }
	  void New_A(wxCommandEvent &event) { new_a->OnEnter(); }

	  void DeleteColor(wxCommandEvent &event) { 
		  colorbox->points.erase(colorbox->points.begin()+colorbox->GetSelection()); 
		  colorbox->setColorMap();
	  }

	  void AddColor(wxCommandEvent &event) {
		  new_t->OnEnter();
		  new_r->OnEnter();
		  new_g->OnEnter();
		  new_b->OnEnter();
		  new_a->OnEnter();
		  colorbox->points.push_back(new_point);	
		  colorbox->setColorMap();
	  }
	  
	  template <class T>
	  string make_str(T t) {
		ostringstream s;
		s << t;
		return s.str();
	  }

	  void SelectColor(wxCommandEvent &event) {
		  int s=colorbox->GetSelection();
		  if(s>=0) {
			  edit_point=colorbox->points[s];  
		  }
		  edit_t->SetValue(make_str(edit_point.first).c_str());
		  edit_r->SetValue(make_str((int)edit_point.second[0]).c_str());
		  edit_g->SetValue(make_str((int)edit_point.second[1]).c_str());
		  edit_b->SetValue(make_str((int)edit_point.second[2]).c_str());
		  edit_a->SetValue(make_str((int)edit_point.second[3]).c_str());
	  }


DECLARE_EVENT_TABLE()
};

//const int CM_COLOR_SEL = 510;
//const int CM_COLOR_DELETE = 511;
//const int CM_COLOR_ADD = 512;

BEGIN_EVENT_TABLE(ColorMapDialog, wxDialog)
	EVT_LISTBOX(CM_COLOR_SEL, ColorMapDialog::SelectColor) 
	EVT_BUTTON(CM_COLOR_DELETE, ColorMapDialog::DeleteColor)
	EVT_BUTTON(CM_COLOR_ADD, ColorMapDialog::AddColor)
	EVT_BUTTON(CM_COLOR_EDIT, ColorMapDialog::EditColor)

	EVT_TEXT_ENTER(CM_EDIT_T, ColorMapDialog::EditColor)
	EVT_TEXT_ENTER(CM_EDIT_R, ColorMapDialog::EditColor)
	EVT_TEXT_ENTER(CM_EDIT_G, ColorMapDialog::EditColor)
	EVT_TEXT_ENTER(CM_EDIT_B, ColorMapDialog::EditColor)
	EVT_TEXT_ENTER(CM_EDIT_A, ColorMapDialog::EditColor)

	EVT_TEXT_ENTER(CM_NEW_T, ColorMapDialog::New_T)
	EVT_TEXT_ENTER(CM_NEW_R, ColorMapDialog::New_R)
	EVT_TEXT_ENTER(CM_NEW_G, ColorMapDialog::New_G)
	EVT_TEXT_ENTER(CM_NEW_B, ColorMapDialog::New_B)
	EVT_TEXT_ENTER(CM_NEW_A, ColorMapDialog::New_A)
END_EVENT_TABLE()

void MyFrame::ChangeColorMap(wxCommandEvent& event)
{
	new ColorMapDialog(this,m_canvas->p_gen);
//	win_tools::fyi_msg("not implemented");
}


template <class T, class container>
T * MakeArray(const container &c) {
	T * rval = new T[c.size()];
	int a=0;
	for(container::const_iterator i = c.begin();i!=c.end() && a<max;i++) {
		rval[a++]=*i;
	}
	return rval;
}	

///////////////////////////////////////////////////////
//Tweak Bitmap stuff
///////////////////////////////////////////////////////
class TweakBitmapDialog : public wxDialog {
	TestGLCanvas * glFrame;
	wxStaticBitmap * current_bmp;

public:
	TweakBitmapDialog(wxWindow * parent, TestGLCanvas * glFrame) :
	  glFrame(glFrame),
	  wxDialog(parent,-1,"Select Bitmap", wxPoint(0,10), wxSize(400,300)) {

		string texture_file=glFrame->p_gen->get_texture_file();
		current_bmp = new wxStaticBitmap(this, -1, iflBitmap(texture_file.c_str()), 
			wxPoint(50,30),wxDefaultSize);

		wxArrayString image_names;
		wxDir::GetAllFiles(wxGetCwd() + "\\particle_images", &image_names,wxEmptyString,wxDIR_FILES );
		wxString * str_array = new wxString[image_names.GetCount()];
		int sel;
		for(int i=0;i<image_names.GetCount();i++) {
			str_array[i]=image_names[i].AfterLast('\\');
			if(string("particle_images\\")+str_array[i].c_str()==texture_file) {
				sel=i;
			}
		}

		wxChoice * bitmaps = new wxChoice(this, ID_CHOOSE_BITMAP, wxPoint(50,0), 
			wxSize(140,20), image_names.GetCount(), str_array);
		bitmaps->SetSelection(sel);

		delete [] str_array;


		Show();	  
	  }

	  void OnChoose(wxCommandEvent &event) {
			wxString newBmp = event.GetString();
			newBmp="particle_images\\"+newBmp;
			if(newBmp) {
				glFrame->p_gen->set_texture(newBmp.c_str());
				current_bmp->SetBitmap(iflBitmap(newBmp));
			}
	  }

DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(TweakBitmapDialog, wxDialog)
	EVT_CHOICE(ID_CHOOSE_BITMAP, TweakBitmapDialog::OnChoose)
END_EVENT_TABLE()

void MyFrame::TweakBitmap(wxCommandEvent& event)
{
	(void) new TweakBitmapDialog(this,m_canvas);
}



///////////////////////////////////////////////////////
//Tweak Param stuff
///////////////////////////////////////////////////////
class TweakParamsDialog : public wxDialog {
	effects::StandardEmitter<> * emitter;
	
	wxTextCtrl * life;
	wxTextCtrl * dir;
	wxTextCtrl * emit_rate;
	wxTextCtrl * speed;
	wxTextCtrl * force;


public:
	  //ug, really should have avoided this bit of cut & paste...

	
	TweakParamsDialog(wxWindow * parent, effects::StandardEmitter<> * emitter) :
	  emitter(emitter),
	  wxDialog(parent,-1,"Param Editor", wxPoint(0,10), wxSize(150,130)) {

		life = new wxTextCtrl( this, PARAM_LIFESPAN, "", wxPoint(50, 0), wxSize(80, -1), wxTE_PROCESS_ENTER  );		
		(*life) << emitter->max_age;
		(void) new wxStaticText(this, -1, "lifespan:", wxPoint(5,5));

		dir = new wxTextCtrl( this, PARAM_DIR, "", wxPoint(50, 20), wxSize(80, -1), wxTE_PROCESS_ENTER  );		
		(*dir) << emitter->dir*RAD2DEG;
		(void) new wxStaticText(this, -1, "direction:", wxPoint(5,25));


		emit_rate = new wxTextCtrl( this, PARAM_EMIT_RATE, "", wxPoint(50, 40), wxSize(80, -1), wxTE_PROCESS_ENTER  );		
		(*emit_rate) << emitter->emit_rate;
		(void) new wxStaticText(this, -1, "emit rate:", wxPoint(5,45));


		speed = new wxTextCtrl( this, PARAM_SPEED, "", wxPoint(50, 60), wxSize(80, -1), wxTE_PROCESS_ENTER  );		
		(*speed) << emitter->speed;
		(void) new wxStaticText(this, -1, "speed:", wxPoint(5,65));


		force = new wxTextCtrl( this, PARAM_FORCE, "", wxPoint(50, 80), wxSize(80, -1), wxTE_PROCESS_ENTER  );		
		(*force) << emitter->g;
		(void) new wxStaticText(this, -1, "force:", wxPoint(5,85));


		Show();	  
	  }

	  void EnterLife(wxCommandEvent &event) {
		using namespace boost_ex;
		string v(life->GetValue().c_str());

		if( is_int(v) ) {
			emitter->max_age=make_int(v);
		}
		else {
			life->SetValue("");
			(*life) << emitter->max_age;
		}
	  }

	  void EnterEmitRate(wxCommandEvent &event) {
		using namespace boost_ex;
		string v(emit_rate->GetValue().c_str());

		if( is_int(v) ) {
			emitter->emit_rate=make_int(v);
		}
		else {
			emit_rate->SetValue("");
			(*emit_rate) << emitter->emit_rate;
		}
	  }

	  void EnterSpeed(wxCommandEvent &event) {
		using namespace boost_ex;
		string v(speed->GetValue().c_str());

		if( is_float(v) ) {
			emitter->speed=make_float(v);
		}
		else {
			speed->SetValue("");
			(*speed) << emitter->speed;
		}
	  }


	  void EnterForce(wxCommandEvent &event) {
		using namespace boost_ex;
		string v(force->GetValue().c_str());

		if( is_float(v) ) {
			emitter->g=make_float(v);
		}
		else {
			force->SetValue("");
			(*force) << emitter->g;
		}
	  }

	  void EnterDir(wxCommandEvent &event) {
		using namespace boost_ex;
		string v(dir->GetValue().c_str());

		if( is_float(v) ) {
			emitter->dir=make_float(v)*DEG2RAD;
		}
		else {
			dir->SetValue("");
			(*dir) << emitter->dir*RAD2DEG;
		}
	  }



DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(TweakParamsDialog, wxDialog)
	EVT_TEXT_ENTER(PARAM_LIFESPAN, TweakParamsDialog::EnterLife)
	EVT_TEXT_ENTER(PARAM_DIR, TweakParamsDialog::EnterDir)
	EVT_TEXT_ENTER(PARAM_EMIT_RATE, TweakParamsDialog::EnterEmitRate)
	EVT_TEXT_ENTER(PARAM_SPEED, TweakParamsDialog::EnterSpeed)
	EVT_TEXT_ENTER(PARAM_FORCE, TweakParamsDialog::EnterForce)
END_EVENT_TABLE()

void MyFrame::TweakParams(wxCommandEvent& event)
{
	(void) new TweakParamsDialog(this,m_canvas->p_gen);
}

/*
string file;
istream_to_string(in,file);
load_string(file, append);
*/

namespace effects {


bool load_float(float &f, string::const_iterator &start, string::const_iterator finish) {
	using namespace boost_ex;
	string_results what;
	if(!regex_search(start,finish,what,
	regex("(-?\\d+(\\.\\d+)?)|(\\.\\d+)"))) return false;
	f=make_float(string(what[0].first,what[0].second));
	start=what[0].second;
	return true;
}

bool load_int(int &i, string::const_iterator &start, string::const_iterator finish) {
	using namespace boost_ex;
	string_results what;
	if(!regex_search(start,finish,what,
	regex("-?\\d+"))) return false;
	i=make_int(string(what[0].first,what[0].second));
	start=what[0].second;
	return true;
}

bool load_quote(string &s, string::const_iterator &start, string::const_iterator finish) {
	using namespace boost_ex;
	string_results what;
	if(!regex_search(start,finish,what,
	regex("\\\"([^\\\"]*)\\\""))) return false;
	s=string(what[1].first,what[1].second);
	start=what[0].second;
	return true;
}


Linear_Color_Map load_Linear_Color_Map(string::const_iterator &start, string::const_iterator finish) {
	using namespace boost_ex;

	string_results what;
	regex_search(start,finish,what,regex("<color_map>"));
	start=what[0].second;
	regex_search(start,finish,what,regex("<\\\\color_map>"));
	string::const_iterator m_end=what[0].second;

	vector<cmap_point> points;
	while(1) {
		cmap_point add;
		if(!load_float(add.first,start,m_end))
			break;
		if(!load_float(add.second[0],start,m_end))
			break;
		if(!load_float(add.second[1],start,m_end))
			break;
		if(!load_float(add.second[2],start,m_end))
			break;
		if(!load_float(add.second[3],start,m_end))
			break;
		points.push_back(add);
	}
	
	start=m_end;

	return Linear_Color_Map(points);
}

StandardEmitter<Linear_Color_Map> load_StandardEmitter(string::const_iterator &start, string::const_iterator finish) {
	using namespace boost_ex;
	StandardEmitter<Linear_Color_Map> rval;


	string_results what;
	regex_search(start,finish,what,regex("<standard_emitter>"));
	start=what[0].second;
	regex_search(start,finish,what,regex("<\\\\standard_emitter>"));
	string::const_iterator e_end=what[0].second;

	while(1) {
		if(!regex_search(start,e_end,what,regex("(\\S*?):"))) break;
		start=what[0].second;
		string code(what[1].first,what[1].second);
		
		if(code=="dir") load_float(rval.dir,start,e_end);
		else if(code=="g") load_float(rval.g,start,e_end);
		else if(code=="speed") load_float(rval.speed,start,e_end);
		else if(code=="color_fun") rval.color_fun=load_Linear_Color_Map(start,e_end);
		else if(code=="emit_rate") load_int(rval.emit_rate,start,e_end);
		else if(code=="texture_file") {
			//win_tools::fyi_msg("bob");
			string tex;
			load_quote(tex,start,e_end);
			rval.set_texture(tex);
			//win_tools::fyi_msg(file.c_str());
		}
	}
	
	start=e_end;
	return rval;
}

StandardEmitter<Linear_Color_Map> load_StandardEmitter(const char *fname) {
	string file;
	ifstream in(fname);
	boost_ex::istream_to_string(in,file);
	string::const_iterator start=file.begin();
	return load_StandardEmitter(start,file.end());
}

}

void MyFrame::SPE_Import(wxCommandEvent& event) {
	wxFileDialog dialog(this,
                        _T("Import Standard Paticle Emitter Info"),
                        _T(""),
                        _T(""),
                        _T("Standard Particle Emitter (*.spe)|*.spe"),
                        wxOPEN);

    if (dialog.ShowModal() == wxID_OK) {

		*m_canvas->p_gen = effects::load_StandardEmitter(dialog.GetPath());
		//out << *m_canvas->p_gen << endl;
	}
}

void MyFrame::SPE_Export(wxCommandEvent& event) {
	wxFileDialog dialog(this,
                        _T("Export Standard Paticle Emitter Info"),
                        _T(""),
                        _T("my_emitter.spe"),
                        _T("Standard Particle Emitter (*.spe)|*.spe"),
                        wxSAVE);

    if (dialog.ShowModal() == wxID_OK) {
		ofstream out(dialog.GetPath());
		out << *m_canvas->p_gen << endl;
	}

}





