//This program was created using Julian Smart's GL Cube program as a base.

#ifndef _WX_CUBE_H_
#define _WX_CUBE_H_

#include "wx/glcanvas.h"
#include <particle.h>

// Define a new application type
class MyApp: public wxApp
{
public:
    bool OnInit(void);
};

// Define a new frame type
class TestGLCanvas;
class MyFrame: public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
            const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);
    void OnPrint(wxCommandEvent& event);

	void SPE_Export(wxCommandEvent& event);
	void SPE_Import(wxCommandEvent& event);
    void ChangePersp(wxCommandEvent& event);
    void TweakBitmap(wxCommandEvent& event);
    void TweakParams(wxCommandEvent& event);
	void ChangeColorMap(wxCommandEvent& event);


public:
    TestGLCanvas*    m_canvas;

DECLARE_EVENT_TABLE()
};

/*
struct ParticleGenerator {
	virtual void refresh()=0;
	virtual void draw()=0;
	virtual ~ParticleGenerator() {}
	virtual void changeBitmap(wxString newBitmap) {}
};

template <class PSwarm>
struct ParticleTemplate : public PSwarm, public ParticleGenerator {
	void refresh() { PSwarm::refresh(); }
	void draw() { PSwarm::flat_draw(); }
	virtual void changeBitmap(wxString newBitmap) {
		texture=nehe::neheTexture(newBitmap.c_str());
	}
	virtual ~ParticleTemplate() {}
};
*/

class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:
	effects::StandardEmitter<> * p_gen;

 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas", int *attribList=0);
   
 ~TestGLCanvas(void);

 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 void OnIdle( wxIdleEvent& event );

 void Render( void );
 void InitGL(void);
 void gl_draw();

 void setPersp(const int p);
 void gl_print_screen(const char * fname);

private:
  bool   m_init;
  GLuint m_gllist;
  long   m_rleft;
  long   m_rright;

  static unsigned long  m_secbase;
  static int            m_TimeInitialized;
  static unsigned long  m_xsynct;
  static unsigned long  m_gsynct;
 
  long           m_Key;
  unsigned long  m_StartTime;
  unsigned long  m_LastTime;
  unsigned long  m_LastRedraw;

DECLARE_EVENT_TABLE()
};


wxMenu * StandardEmitterMenu();

//Used when we switch around the perspective matrices
extern const int PERSP_MATRIX_8x6;
extern const int PERSP_MATRIX_4x3;
const int PERPS_MATRICES = 2;

extern const wxString PERSP_CHOICES[];




//main menu stuff
//const int ID_FILE_SAVE = 130;
//const int ID_FILE_LOAD = 131;
const int ID_PRINT_SCREEN = 132;
const int ID_TWEAK_CHANGE_TEMPLATE = 133;

//particle tweak menu options
const int ID_CHANGE_PERSP = 150;
const int ID_CHOOSE_BITMAP = 151;
const int ID_TWEAK_PARAMS = 152;
const int ID_TWEAK_BITMAP = 153;
const int ID_TWEAK_COLOR_MAP = 154;
const int ID_EXPORT_SPE = 155;
const int ID_IMPORT_SPE = 156;



#endif

